--[[
Enables the use of custom traffic on a map without the need to edit the actual map files.

Author:     w33zl / WZL Modding (https://github.com/w33zl)
Version:    1.0.0
Modified:   2024-03-17

Changelog:
    1.0.0 - Initial ModHub Release
    0.9.0 - Private beta release

USAGE:

    1. Rename the file 'disabled_customTrafficSystem.xml' to 'customTrafficSystem.xml', the file isrunning found in the 'modSettings/FS22_000_CustomTrafficSystem' folder found in you MyDocuments/MyGames/FarmingSimulator2022 folder.
    2. Edit the <vehicle filename="..." /> XML nodes with your custom traffic (a copy of a default car from the base game is included as a example)
    3. Add the corrensponding .xml, .i3d and .dds files for your traffic in the same folder as the 
    4. Enable the mod and enjoy!

TIP:    You can easily extract vehicles from existing maps and put into your own tailored traffic system in your modSettings folder.
        Just copy the .xml, .i3d and .dds files from the map into your 'modSettings/FS22_000_CustomTrafficSystem' folder and add a <vehicle /> XML node in the 'customTrafficSystem.xml' file.
        Ensure that the path in the vehicle .xml files, i.e. the <assets /> node, are relative to the 'customTrafficSystem.xml' file. 

For more information, see the README.txt file.

]]

local CUSTOM_TRAFFIC_FILENAME = "customTrafficSystem"
local DISABLED_TRAFFIC_SYSTEM_PREFIX = "disabled_"
local DEFAULT_TRAFFICSYSTEM_FILENAME = "default_customTrafficSystem.xml"
local DEFAULT_SAMPLEVEHICLE_FILENAME = "vehicle_04.xml"

local g_currentModDirectory = g_currentModDirectory
local g_modSettingsDirectory = g_modSettingsDirectory
local g_currentModSettingsDirectory = g_currentModSettingsDirectory

local customTrafficSystemXmlFilename = g_currentModSettingsDirectory .. CUSTOM_TRAFFIC_FILENAME .. ".xml"

FSBaseMission.loadMap = Utils.prependedFunction(FSBaseMission.loadMap, function(baseMission, ...)
    Log:debug("FSBaseMission.loadMap")

    createFolder(g_currentModSettingsDirectory)

    local disabledTrafficSystemXmlFilename = g_currentModSettingsDirectory .. DISABLED_TRAFFIC_SYSTEM_PREFIX .. CUSTOM_TRAFFIC_FILENAME .. ".xml"
    local sampleVehicleXmlFilename = g_currentModSettingsDirectory .. "vehicle_04.xml"
    local xmlAssetsFolder = g_currentModDirectory .. "xml/"

    Log:var("xmlAssetsFolder", xmlAssetsFolder)
    Log:var("sampleVehicleXmlFilename", sampleVehicleXmlFilename)
    Log:var("disabledTrafficSystemXmlFilename", disabledTrafficSystemXmlFilename)

    if not fileExists(disabledTrafficSystemXmlFilename) and not fileExists(customTrafficSystemXmlFilename) then
        Log:debug("Copying default traffic system from '%s' to '%s'", xmlAssetsFolder .. DEFAULT_TRAFFICSYSTEM_FILENAME, disabledTrafficSystemXmlFilename)
        copyFile(xmlAssetsFolder .. DEFAULT_TRAFFICSYSTEM_FILENAME, disabledTrafficSystemXmlFilename, false)
    end

    if not fileExists(sampleVehicleXmlFilename) then
        Log:debug("Copying default sample vehicle from '%s' to '%s'", xmlAssetsFolder .. DEFAULT_SAMPLEVEHICLE_FILENAME, sampleVehicleXmlFilename)
        copyFile(xmlAssetsFolder .. DEFAULT_SAMPLEVEHICLE_FILENAME, sampleVehicleXmlFilename, false)
    end

    local readmeFilename = g_currentModSettingsDirectory .. "README.txt"
    copyFile(g_currentModDirectory .. "/README.txt", readmeFilename, true)

end)

TrafficSystem.load = Utils.overwrittenFunction(TrafficSystem.load, function(self, superFunc, xmlFilename, transformId, useHighProfile, isServer, isClient)
    Log:var("Original traffic system XML filename", xmlFilename)

    local savegameIndex = g_currentMission.missionInfo.savegameIndex
    local savegameSpecificTrafficSystemXmlFilename = customTrafficSystemXmlFilename:gsub(".xml", "_savegame" .. savegameIndex .. ".xml")

    if fileExists(savegameSpecificTrafficSystemXmlFilename) then
        Log:info("Loading Custom Traffic System for savegame #%d from '%s'", savegameIndex, savegameSpecificTrafficSystemXmlFilename)
        xmlFilename = savegameSpecificTrafficSystemXmlFilename
    elseif fileExists(customTrafficSystemXmlFilename) then
        Log:info("Loading Custom Traffic System from '%s'", customTrafficSystemXmlFilename)
        xmlFilename = customTrafficSystemXmlFilename
    else
        Log:warning("Custom Traffic System mod enabled, but no custom traffic system XML data found at path '%s'", customTrafficSystemXmlFilename)
    end

    Log:var("Final traffic system XML filename", xmlFilename)

    return superFunc(self, xmlFilename, transformId, useHighProfile, isServer, isClient)
end)